package com.sintecmedia.components.renderkit;

import java.io.IOException;
import java.util.*;

import javax.faces.component.UIComponent;
import javax.faces.context.ExternalContext;
import javax.faces.context.FacesContext;
import javax.faces.convert.Converter;
import javax.faces.convert.ConverterException;
import javax.faces.convert.DateTimeConverter;

import org.ajax4jsf.renderkit.HeaderResourcesRendererBase;

import com.sintecmedia.components.component.UIWeeklyCalendar;
import com.sintecmedia.components.datamodels.WeeklyCalendarModel;
import com.sintecmedia.components.datamodels.impl.WeeklyCalendarModelDayImpl;
import com.sintecmedia.components.datamodels.impl.WeeklyCalendarModelImpl;

public abstract class WeeklyCalendarRendererBase  
		extends HeaderResourcesRendererBase {

	@SuppressWarnings("unchecked")
	public void decode( FacesContext context, UIComponent component ) {
		ExternalContext external = context.getExternalContext();
		Map requestParams = external.getRequestParameterMap();
		UIWeeklyCalendar weeklyCalendar = (UIWeeklyCalendar)component;
		String clientId = weeklyCalendar.getClientId( context );
		String submittedValue = (String)requestParams.get( clientId );
		
		if (submittedValue != null) {
			weeklyCalendar.setSubmittedValue(submittedValue);
		}
	}	

	protected String getValueAsString( 
			FacesContext context, UIComponent component ) throws IOException {

		UIWeeklyCalendar weeklyCalendar = (UIWeeklyCalendar)component;
		String valueString = (String) weeklyCalendar.getSubmittedValue();

		if (valueString == null) {
			Object value = weeklyCalendar.getValue();
			if (value != null) {
				Converter converter = getConverter(context, weeklyCalendar);
				valueString = converter.getAsString(context, component, value);
			}
		}
		return valueString;
	}
			
	
	public Object getConvertedValue(
			FacesContext context, UIComponent component, Object submittedValue ) 
			throws ConverterException {
		UIWeeklyCalendar weeklyCalendar = (UIWeeklyCalendar)component;
		Converter converter = getConverter( context, weeklyCalendar );
		String valueString = (String)submittedValue;
		return converter.getAsObject( context, component, valueString );
	}
	
	private Converter getConverter( 
			FacesContext context, UIWeeklyCalendar weeklyCalendar )	{
		Converter converter = weeklyCalendar.getConverter();
		if( converter == null ) {
			DateTimeConverter datetime = new DateTimeConverter();
			datetime.setLocale( context.getViewRoot().getLocale() );
			datetime.setTimeZone( TimeZone.getDefault() );
			datetime.setType( "date" );
			datetime.setDateStyle( "medium" );
			datetime.setPattern( "d/m/y" );
			converter = datetime;
		}
		return converter;
	}
	
	/**
	 * 
	 * @param component UIWeeklyCalendar component used.
	 * @return list of days from component data model as it called in default java.util.Calendar intance
	 */
	public List<String> getDaysOfWeek( UIComponent component ) {
		WeeklyCalendarModel model = //null; 
			component != null && component instanceof UIWeeklyCalendar ? 
				((UIWeeklyCalendar)component).getDataModel() : null;
				
		//Days map returned from calendar relays on day's name as key, so it
		//needs refinement by day's index.
		Map<String, Integer> daysMap = 
			Calendar.getInstance().getDisplayNames(
					Calendar.DAY_OF_WEEK, Calendar.LONG, Locale.ENGLISH );
		String[] names = new String[ daysMap.size() ]; 
		for( String name : daysMap.keySet() ) {
			names[ daysMap.get( name ) - 1 ] = name;
		}			
		
		if( model == null ) {
			return Arrays.asList( names );
		}
		
		int startDay = model.getStartDay();
		int endDay = model.getEndDay();
		int size = endDay - startDay + 1;
		List<String> result = new ArrayList<String>( size );
		for( int i = startDay; i <= endDay; i++ ) {			
			result.add( names[i] );
		}
		return result;
	}

	/**
	 * 
	 * @param component UIWeeklyCalendar component used.
	 * @return list of hours for start of each time slot in component data model
	 */
	public List<String> getHoursList( UIComponent component ) {
		WeeklyCalendarModel model = //null;
			component != null && component instanceof UIWeeklyCalendar ? 
				((UIWeeklyCalendar)component).getDataModel() : null;
		int index, finish, slotSize;
		if( model != null ) {
			index = model.getStartTime();
			finish = model.getEndTime();
			slotSize = model.getSlotSize();
		} else {
			index = WeeklyCalendarModelDayImpl.DEF_START_TIME;
			finish = WeeklyCalendarModelDayImpl.DEF_END_TIME;
			slotSize = WeeklyCalendarModelDayImpl.DEF_SLOT_SIZE; 

		}
		
		List<String> result = new ArrayList<String>();
		int currHour;
		int currMinute;
		for( ;index < finish; index = index + slotSize ) {
			currHour = index / 60;
			currMinute = index % 60;			
			result.add(	
				( currHour < 10 ? "0" : "" ) + currHour + ":" + 
				( currMinute < 10 ? "0" : "" ) + currMinute );			
		}
		return result;
	}
	
	/**
	 * 
	 * @param dayName name of day as it called in default java.util.Calendar instance
	 * @return index of day specified by parameter
	 */
	public List<Integer> getDayIndexes( UIComponent component ) {
		WeeklyCalendarModel model = //null; 
			component != null && component instanceof UIWeeklyCalendar ? 
				((UIWeeklyCalendar)component).getDataModel() : null;
		int startDay, endDay;
		if( model != null ) {
			startDay = model.getStartDay();
			endDay = model.getEndDay();
		} else {
			startDay = WeeklyCalendarModelImpl.DEF_START_DAY;
			endDay = WeeklyCalendarModelImpl.DEF_END_DAY;
		}
		int size = endDay - startDay + 1;
		List<Integer> result = new ArrayList<Integer>( size );
		for( int i = startDay; i <= endDay; i++ ) {			
			result.add( i );
		}
		return result;
	}
}

